<?php
#####################################################################################################
#
#					Module pour la plateforme de paiement PayZen
#						Version : 2.13b (révision 29800)
#									########################
#					Développé pour Magento
#						Version : 1.3.2.2
#						Compatibilité plateforme : V1
#									########################
#					Développé par Lyra Network
#						http://www.lyra-network.com/
#						10/10/2011
#						Contact : support@payzen.eu
#
#####################################################################################################

class Mage_Vads_StandardController extends Mage_Core_Controller_Front_Action
{
	/**
	 * var used in log()
	 * @var string
	 */
	var $_currentMethod;
	
	/**
	 * Return the status of the order to set while the client is on the payment gateway.
	 * It will return the status 'pending_vads' if it exists (automatically created in magento 1.3/1.4, has to be created by hand in magento 1.5)
	 * else the default "pending payment" status
	 */
	private function getStatusPendingPayment() {
    	/* @var $stateStatusInfo Mage_Sales_Model_Order_Config */
    	$stateStatusInfo = Mage::getModel('sales/order_config');
    	if(array_key_exists('pending_vads', $stateStatusInfo->getStatuses())) {
    		return 'pending_vads';
    	} else { 
    		return $stateStatusInfo->getStateDefaultStatus(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT);
    	}
	}
    
    /**
     * Returns the url matching the $url argument. If $url is not already an absolute url, treats it as a magento path.
     * @param string $url
     * @param int $store_id
     */
    private function prepareUrl($url,$store_id) {
    	$result = '';
    	// Preserve absolute urls
    	if(strpos($url,'http')===0) {
    		$result = $url;
    	}
    	else {
    		// transform path to url
    		$result = Mage::getUrl($url,array('_secure'=>true,'_store'=>$store_id));
    	}
    	$this->log("prepareUrl : $url => $result");
    	return $result;
    }

	/**
	 * Redirect customer to the client gateway
	 */
    public function paymentAction() {
    	$this->_currentMethod = __METHOD__;
    	$this->log('Start');

    	// Load session
    	$session = Mage::getSingleton('checkout/session');
    	
    	$session->setVadsStandardQuoteId($session->getQuoteId());

    	// Clear redirect url : it is not useful anymore and may be called from unwanted locations (e.g. cart/add with out of stock products...)
    	$session->unsRedirectUrl();

		// Load order
    	$order_id = $session->getLastRealOrderId();
    	/* @var $order Mage_Sales_Model_Order */
    	$order = Mage::getModel('sales/order');
		$order->loadByIncrementId($order_id);

		// Check No risk
		if(!$order->getId()) {
			$this->log("Payment attempt for an unknown order id - interrupting process.", Zend_Log::WARN);
			Mage::throwException($this->__("No order for processing found"));
		}
    	
		// Check No risk 2 : check vs. the client who had no cart...
    	if ($order->getTotalDue()==0) {
    		$this->log("Payment attempt with no amount - redirecting to cart.");
            $this->_redirect('checkout/cart');
            return;
        }
		
    	// ------------
    	// Display form
    	// ------------
    	// Set config parameters
    	$this->log('Initialize payment parameters');
    	/* @var $standard Mage_Vads_Model_Method_Standard */
    	$standard = Mage::getModel('vads/method_standard');

    	$config_fields = array('platform_url','key_test','key_prod','capture_delay','ctx_mode','site_id',
			'validation_mode','redirect_enabled','redirect_success_timeout','redirect_success_message',
    		'redirect_error_timeout','redirect_error_message','return_mode'
    	);
    	foreach($config_fields as $field)
    	{
    		$standard->set($field, $standard->getConfigData($field));
    	}
    	
    	// Set return url (build it if only path has been configured)
    	$standard->set('url_return',$this->prepareUrl($standard->getConfigData('url_return'), $order->getStore()->getId()));

    	// Set misc parameters
    	$standard->set('version','V1');
    	$cards = explode(",",$standard->getConfigData('payment_cards'));
		$cards = in_array("",$cards) ? "" : implode(";",$cards);
		$standard->set('payment_cards',$cards);	// payment_cards is given as csv by magento
    	$standard->set('payment_config', 'SINGLE');
    	$standard->set('cust_email',$order->getCustomerEmail());
    	$standard->set('cust_id',$order->getCustomerId());
    	$standard->set('cust_name',$order->getBillingAddress()->getName());
    	$standard->set('cust_address',$order->getBillingAddress()->getStreet(1));
    	$standard->set('cust_zip',$order->getBillingAddress()->getData('postcode'));
    	$standard->set('cust_city',$order->getBillingAddress()->getData('city'));
    	$standard->set('cust_country',$order->getBillingAddress()->getData('country_id'));
    	$standard->set('cust_phone',$order->getBillingAddress()->getData('telephone'));
    	$standard->set('contrib','Magento_1.3.2.2_2.13b');

    	// Set order_id
    	$standard->set('order_id',$order_id);

    	// Set the amount to pay
		$amount = round($order->getTotalDue()*100);
		$standard->set('amount',$amount);
		
		// Set currency
		$currency = $standard->vads_api->convertAlphaCurrency($order->getOrderCurrency()->getCode());
		if(!$currency) {
			$this->log("Could not find currency numeric code for currency : ".$order->getOrderCurrency()->getCode());
			Mage::throwException($this->__("Unsupported currency"));
		}
    	$standard->set('currency',$currency);

    	// Set the language code
    	$default_lang = $standard->getConfigData("language");
    	$current_lang = substr(Mage::app()->getLocale()->getLocaleCode(), 0, 2);
    	$available_lang = $standard->getConfigArray('languages');
		if( in_array($current_lang,$available_lang) ) {
			$lang = $current_lang;
		}
		else {
			$lang = $default_lang;
		}
    	$standard->set('language',$lang);

    	// redirect to gateway
    	$this->log('Display form and javascript');
    	$response  = '<html><head><title>Redirection</title></head><body>';
    	$response .= $standard->getRequestHtmlForm();
    	$response .= '<script type="text/javascript">document.forms[0].style.display=\'none\'; document.forms[0].submit();</script></body></html>';
    	$this->getResponse()->setBody($response);

    	// ------------------------------------
    	// Put order on "pending payment" state
        // ------------------------------------
        $this->log("Client ".$order->getCustomerEmail().' sent to payment page for order '.$order->getId(), Zend_Log::INFO);
        $this->log('Parameters : '.$standard->vads_api->getRequestUrlEncodedFields());
		$order->setState(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT, $this->getStatusPendingPayment(),
			$this->__("pending payment, client sent to vads gateway"));
		$order->save();

		$this->log('End');
		return;
    }

    /**
     * Action called after the client returns from payment gateway
     */
    public function returnAction()
    {
    	$this->_currentMethod = __METHOD__;
    	$this->log('Start');
    	$req = $this->getRequest()->getParams();
    	if(stripos($req['payment_config'],'MULTI') !== false || stripos($req['contrib'],'multi') !== false)	{
			// multi-payment, load appropriate module
			$this->log('Redirect to multi payment module');
			$this->_redirect('vadsmulti/standard/return',$req);
			return;
		}
		else {
			// simple payment, fine
		}

    	// Load standard model to analyse response
		$this->log('Loading vads api');
    	$standard = Mage::getModel('vads/method_standard');
		$standard->setResponseFromPost(
			$req,
			$standard->getConfigData('key_test'),
			$standard->getConfigData('key_prod'),
			$standard->getConfigData('ctx_mode')
		);

		if( ! $standard->isAuthentifiedResponse() )
		{
			// authentification failed
			$this->log($standard->getIpAddress() . $this->__(" tries to access our vads/standard/return page without valid signature. It may be a hacking attempt."),
					Zend_Log::WARN);
			$this->_redirect('checkout/onepage/failure');
			$this->log('End');
			return;
		}
		$this->log('Request authenticated');

		// Load session
		$session = Mage::getSingleton('checkout/session');
		$session->setQuoteId($session->getVadsStandardQuoteId(true));

		// Load order
		/* @var $order Mage_Sales_Model_Order */
		$order = Mage::getModel('sales/order');
		$order->loadByIncrementId( $req['order_id'] );
		
		if( $order->getStatus() == $this->getStatusPendingPayment() )
		{
			$this->log('Order '.$order->getId().' is waiting payment');
			
			// Save Platform responses
			$order->getPayment()
	        	->setAmount($req['amount'] / 100)
	        	->setCcTransId($req['auth_number'])
	        	->setLastTransId($req['trans_id'])
	        	->setCcType($req['card_brand'])
	        	->setCcNumberEnc($req['card_number'])
	        	->setStatus($req['result'])
	        	->setStatusDescription($standard->getResponseMessage('detail'))

	        	->setTransactionId($req['trans_id']);	
			
			// Order waiting for payment
			if($standard->isAcceptedPayment())
			{
				$this->log('Payment for order '.$order->getId().' has been confirmed by client return. <!> This means the check url did not work.',
						Zend_Log::WARN);
				// Register the payment
				$order->addStatusToHistory($order->getStatus(), $standard->getResponseMessage('detail'));
				// Create invoice
				$this->registerOrder($order);

				// Display success page
				if( $standard->getConfigData('ctx_mode') == 'TEST' )
				{
					// order not paid => not validated from check url ; ctx_mode=TEST => user is webmaster
					// So log and display a warning about check url not working
					$message = $this->__("Have you properly configured the check url in your bank backoffice ?");
					$session->addError($message);
				}

				$this->_redirect('checkout/onepage/success', array('_store'=>$order->getStore()->getId()));
			}
			else {
				// Client returns with a cancelled/refused payment, send him back to checkout
				$this->log('Payment for order '.$order->getId().' has failed.', Zend_Log::INFO);
				$this->manageRefusedPayment($order, false);
			}
		}
		else
		{
			$this->log('Order '.$order->getId().' has already been registered.');
			// Payment already registered
			if($standard->isAcceptedPayment()) {
				$this->log('Order '.$order->getId().' is reconfirmed');
				// Deactivate quote
				Mage::getSingleton('checkout/session')->getQuote()->setIsActive(false)->save();
				$this->_redirect('checkout/onepage/success', array('_store'=>$order->getStore()->getId()));
			}
			elseif($order->getState() === Mage_Sales_Model_Order::STATE_CANCELED) {
				$this->log('Order '.$order->getId().' cancelation is reconfirmed');
				$this->manageRefusedPayment($order, false);
			}
			else {
				// This is an error case, the client returns with an error code but the payment already has been accepted
				$this->log('Order '.$order->getId().' has been validated but we receive a payment error code !',
						Zend_Log::ERR);
				$message = $this->__("Received error code but the order has already been accepted");
				$session->addError($this->__($message));
				$this->_redirect('checkout/onepage/failure', array('_store'=>$order->getStore()->getId()));
			}
		}
		$this->log('End');
		return;
    }


    /**
     * Action called by the payment gateway to confirm (or not) a payment
     */
    public function checkAction() {
    	$this->_currentMethod = __METHOD__;
    	$this->log('Start');
    	$req = $this->getRequest()->getParams();
    	if(stripos($req['payment_config'],'MULTI') !== false || stripos($req['contrib'],'multi') !== false)	{
			$this->log('Redirect to multi payment module');
			// multi-payment, load appropriate module
			$query_string = '?';
			foreach($req as $k=>$v) {
				$query_string .= $k.'='.urlencode($v).'&';
			}
			$query_string = substr($query_string,0,-1);
			// We don't use magento built-in _redirect function because of troubles with overslashed urls
			$this->_redirectUrl(Mage::getUrl('vadsmulti/standard/check').$query_string);
			$this->log('End');
			return;
		}
		else {
			// simple payment, fine
		}

		// Load session
		$session = Mage::getSingleton('checkout/session');
		$session->setQuoteId($session->getVadsStandardQuoteId(true));
		
		// Load order
		/* @var $order Mage_Sales_Model_Order */
		$order = Mage::getModel('sales/order');
		$order->loadByIncrementId( $req['order_id'] );
		
		// Get store id from order
		$storeId = $order->getStore()->getId();
		
    	// Load standard model to analyse response
		$this->log('Loading vads api');
    	$standard = Mage::getModel('vads/method_standard');
		$standard->setResponseFromPost(
			$req,
			$standard->getConfigData('key_test', $storeId),
			$standard->getConfigData('key_prod', $storeId),
			$standard->getConfigData('ctx_mode', $storeId)
		);

		if( ! $standard->isAuthentifiedResponse() ) {
			// authentification failed
			$this->log($standard->getIpAddress() . $this->__(" tries to access our vads/standard/return page without valid signature. It may be a hacking attempt."),
					Zend_Log::WARN);
        	$message = $standard->getCheckUrlResponse('auth_fail');
        	$this->getResponse()->setBody($message);
			$this->log('End');
			return;
		}
		$this->log('Request authenticated');

		if( $order->getStatus() == $this->getStatusPendingPayment() ) {
			$this->log('Order '.$order->getId().' is waiting payment');
			
			// Save Platform responses
			$order->getPayment()
	        	->setAmount($req['amount'] / 100)
	        	->setCcTransId($req['auth_number'])
	        	->setLastTransId($req['trans_id'])
	        	->setCcType($req['card_brand'])
	        	->setCcNumberEnc($req['card_number'])
	        	->setStatus($req['result'])
	        	->setStatusDescription($standard->getResponseMessage('detail'))

	        	->setTransactionId($req['trans_id']);
			
			// Order waiting for payment
			if($standard->isAcceptedPayment())
			{
				$this->log('Payment for order '.$order->getId().' has been confirmed by check url', Zend_Log::INFO);
				// Register the payment
				$order->addStatusToHistory($order->getStatus(), $standard->getResponseMessage('detail'));
				// Create invoice
				$this->registerOrder($order);

				// Display confirmation message
				$message = $standard->getCheckUrlResponse('payment_ok');
        		$this->getResponse()->setBody($message);
			}
			else {
				// Gateway refused the payment
				$this->log('Payment for order '.$order->getId().' has been invalidated by check url', Zend_Log::INFO);
				$this->manageRefusedPayment($order, true);
			}
		}
		else {
			// Payment already registered
			if($standard->isAcceptedPayment()) {
				$this->log('Order '.$order->getId().' is reconfirmed');
				$message = $standard->getCheckUrlResponse('payment_ok_already_done');
        		$this->getResponse()->setBody($message);
			}
			elseif($order->getState() === Mage_Sales_Model_Order::STATE_CANCELED) {
				$this->log('Order '.$order->getId().' cancelation is confirmed');
				$this->manageRefusedPayment($order, true);
			}
			else {
				// This is an error case, the client returns with an error code but the payment already has been accepted
				$this->log('Order '.$order->getId().' has been validated but we receive a payment error code !',
						Zend_Log::ERR);
				$message = $standard->getCheckUrlResponse('payment_ko_on_order_ok');
				$this->getResponse()->setBody($message);
			}
		}

		$this->log('End');
		return;
    }


    /**
     *  Save invoice for order
     *
     *  @param    Mage_Sales_Model_Order $order
     *  @return	  boolean Can save invoice or not
     */
    function registerOrder(Mage_Sales_Model_Order $order)
    {
    	$this->_currentMethod = __METHOD__;
    	$this->log('Start');
    	$this->log('Retrieving statuses configuration');
    	$standard = Mage::getModel('vads/method_standard');
    	$newStatus = $standard->getConfigData('registered_order_status', $order->getStore()->getId());
    	/* var $stateStatusInfo Mage_Sales_Model_Order_Config */
    	$stateStatusInfo = Mage::getModel('sales/order_config');
    	$processingStatuses = $stateStatusInfo->getStateStatuses(Mage_Sales_Model_Order::STATE_PROCESSING);

        if (array_key_exists($newStatus, $processingStatuses)) {
			$this->log('Capturing payment for order '.$order->getId());
			
			if($order->canInvoice()) {
				$this->log('Creating invoice for order '.$order->getId());
				$convertor = Mage::getModel('sales/convert_order');

				$invoice = $convertor->toInvoice($order);

				foreach ($order->getAllItems() as $orderItem)
				{
					if (!$orderItem->getQtyToInvoice())
				    {
						continue;
					}

					$item = $convertor->itemToInvoiceItem($orderItem);
					$item->setQty($orderItem->getQtyToInvoice());
					$invoice->addItem($item);
				}

				$invoice->collectTotals();
				$invoice->register()->capture();

				Mage::getModel('core/resource_transaction')
				  ->addObject($invoice)
				  ->addObject($invoice->getOrder())
				  ->save();
				$order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, $newStatus,
	            	Mage::helper('vads')->__('Invoice %s was created', $invoice->getIncrementId()));
			}
			else {
				$order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, $newStatus,
					Mage::helper('vads')->__('Order registered'));
			}
        }
        else {
			$this->log('Capturing payment for order '.$order->getId());
			$order->setState(Mage_Sales_Model_Order::STATE_NEW, $newStatus, Mage::helper('vads')->__('Order registered'));
        }

		$this->log('Saving confirmed order and sending email');
		$order->sendNewOrderEmail();
		$order->save();

		$this->log('End');
        return false;
    }


    /**
     * Cancel order and return a message to the payment gateway / redirect client to cart
     * @param Mage_Sales_Model_Order $order
     * @param boolean $from_server
     */
    function manageRefusedPayment(Mage_Sales_Model_Order $order, $from_server=false) {
    	$this->_currentMethod = __METHOD__;
    	$this->log('Start');
    	$this->log('Canceling order '.$order->getId(), Zend_Log::INFO);
    	
    	$standard = Mage::getModel('vads/method_standard');

    	$order->cancel()->save();

		if($from_server) {
			$this->log('Displaying check url message');
			$message = $standard->getCheckUrlResponse('payment_ko');
			$this->getResponse()->setBody($message);
		}
		else {
	    	/* @var $session Mage_Checkout_Model_Session */
			$this->log('Unsetting order data in session');
			$session = Mage::getSingleton('checkout/session');
			$session->unsLastQuoteId()
				->unsLastSuccessQuoteId()
				->unsLastOrderId()
				->unsLastRealOrderId()
				->addWarning($this->__('PayZen checkout and order have been cancelled.'))
			;
			$this->log('Redirecting to cart');
			$this->_redirect('checkout/cart', array('_store'=>$order->getStore()->getId()));
		}
		$this->log('End');
		return;
    }

	/**
	 * Log function. Uses Mage::log with built-in extra data (module version, method called...)
	 * @param $message
	 * @param $level
	 */
    function log($message, $level=null) {
    	//TODO fonction encore perfectible ($_currentMethod pour fonctions imbriquées...)
		if (!Mage::getStoreConfig('dev/log/active')) {
    		return;
    	}

    	$log  = '';
    	$log .= 'PayZen v2.13b.r29800';
    	$log .= ' - '.$this->_currentMethod;
//    	$log .= ' line '.$line;
    	$log .= ' : '.$message;
		Mage::log($log, $level, 'vads.log');
    }
}
?>